Package: webmockr
Title: Stubbing and Setting Expectations on 'HTTP' Requests
Description: Stubbing and setting expectations on 'HTTP' requests.
    Includes tools for stubbing 'HTTP' requests, including expected
    request conditions and response conditions. Match on
    'HTTP' method, query parameters, request body, headers and
    more. Can be used for unit tests or outside of a testing
    context.
Version: 2.2.0
Authors@R: c(
    person("Scott", "Chamberlain", role = c("aut", "cre"), email =
    "myrmecocystus+r@gmail.com", comment = c(ORCID="0000-0003-1444-9135")),
    person("Aaron", "Wolen", role = "ctb",
    comment = c(ORCID="0000-0003-2542-2202")),
    person("rOpenSci", role = "fnd", comment = c(ROR = "019jywm96"))
    )
License: MIT + file LICENSE
URL: https://github.com/ropensci/webmockr,
        https://books.ropensci.org/http-testing/,
        https://docs.ropensci.org/webmockr/
BugReports: https://github.com/ropensci/webmockr/issues
Encoding: UTF-8
Language: en-US
Depends: R(>= 4.1.0)
Imports: curl, jsonlite, magrittr (>= 1.5), R6 (>= 2.1.3), urltools (>=
        1.6.0), fauxpas, rlang, cli
Suggests: testthat (>= 3.0.0), xml2, crul, httr, httr2, diffobj, withr
RoxygenNote: 7.3.2
Config/testthat/edition: 3
Config/testthat/parallel: true
X-schema.org-applicationCategory: Web
X-schema.org-keywords: http, https, API, web-services, curl, mock,
        mocking, fakeweb, http-mocking, testing, testing-tools, tdd
X-schema.org-isPartOf: https://ropensci.org
NeedsCompilation: no
Packaged: 2025-07-17 17:53:01 UTC; sckott
Author: Scott Chamberlain [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-1444-9135>),
  Aaron Wolen [ctb] (ORCID: <https://orcid.org/0000-0003-2542-2202>),
  rOpenSci [fnd] (ROR: <https://ror.org/019jywm96>)
Maintainer: Scott Chamberlain <myrmecocystus+r@gmail.com>
Repository: CRAN
Date/Publication: 2025-07-21 05:00:02 UTC
Built: R 4.3.3; ; 2025-07-21 06:51:03 UTC; unix
