# Generated by roxygen2: do not edit by hand

S3method(print,mock_file)
S3method(print,partial)
S3method(print,webmockr_config)
export("%>%")
export(CrulAdapter)
export(Httr2Adapter)
export(HttrAdapter)
export(RequestPattern)
export(RequestSignature)
export(Response)
export(build_crul_response)
export(build_httr2_request)
export(build_httr2_response)
export(build_httr_response)
export(disable)
export(enable)
export(enabled)
export(excluding)
export(httr2_mock)
export(httr_mock)
export(including)
export(last_request)
export(last_stub)
export(mock_file)
export(pluck_body)
export(remove_request_stub)
export(request_registry)
export(request_registry_clear)
export(stub_body_diff)
export(stub_registry)
export(stub_registry_clear)
export(stub_request)
export(to_raise)
export(to_return)
export(to_return_)
export(to_timeout)
export(webmockr_allow_net_connect)
export(webmockr_configuration)
export(webmockr_configure)
export(webmockr_configure_reset)
export(webmockr_crul_fetch)
export(webmockr_disable)
export(webmockr_disable_net_connect)
export(webmockr_enable)
export(webmockr_net_connect_allowed)
export(webmockr_reset)
export(wi_th)
export(wi_th_)
importFrom(R6,R6Class)
importFrom(cli,ansi_collapse)
importFrom(cli,cat_line)
importFrom(cli,cli_abort)
importFrom(cli,format_error)
importFrom(fauxpas,HTTPRequestTimeout)
importFrom(jsonlite,base64_enc)
importFrom(jsonlite,fromJSON)
importFrom(jsonlite,toJSON)
importFrom(jsonlite,validate)
importFrom(magrittr,"%>%")
importFrom(rlang,abort)
importFrom(rlang,caller_arg)
importFrom(rlang,caller_env)
importFrom(rlang,check_installed)
importFrom(rlang,has_name)
importFrom(rlang,is_empty)
importFrom(rlang,is_error)
importFrom(rlang,is_function)
importFrom(rlang,is_installed)
importFrom(rlang,is_list)
importFrom(rlang,is_na)
importFrom(rlang,is_null)
importFrom(rlang,try_fetch)
importFrom(rlang,warn)
