#' This function is defunct.
#' @export
#' @rdname webmockr_enable-defunct
#' @keywords internal
webmockr_enable <- function(...) .Defunct("enable")

#' This function is defunct.
#' @export
#' @rdname webmockr_disable-defunct
#' @keywords internal
webmockr_disable <- function(...) .Defunct("disable")

#' This function is defunct.
#' @export
#' @rdname to_return_-defunct
#' @keywords internal
to_return_ <- function(...) .Defunct("to_return")

#' This function is defunct.
#' @export
#' @rdname wi_th_-defunct
#' @keywords internal
wi_th_ <- function(...) .Defunct("wi_th")

#' Defunct functions in \pkg{webmockr}
#'
#' - [webmockr_enable()]: Function removed, see [enable()]
#' - [webmockr_disable()]: Function removed, see [disable()]
#' - [to_return_]: Only [to_return()] is available now
#' - [wi_th_]: Only [wi_th()] is available now
#'
#' @name webmockr-defunct
NULL
