% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/defunct.R
\name{wi_th_}
\alias{wi_th_}
\title{This function is defunct.}
\usage{
wi_th_(...)
}
\description{
This function is defunct.
}
\keyword{internal}
